'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var scheduler = require('scheduler');
var useContextSelector = require('use-context-selector');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _createForOfIteratorHelperLoose(o, allowArrayLike) {
  var it;

  if (typeof Symbol === "undefined" || o[Symbol.iterator] == null) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;
      return function () {
        if (i >= o.length) return {
          done: true
        };
        return {
          done: false,
          value: o[i++]
        };
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  it = o[Symbol.iterator]();
  return it.next.bind(it);
}

var isSSR = typeof window === 'undefined' || /ServerSideRendering/.test(window.navigator && window.navigator.userAgent);
var useIsoLayoutEffect = isSSR ? function (fn) {
  return fn();
} : React.useLayoutEffect;

var MAX_CHAIN_LENGTH = 10;
var SMALL_MAP_SIZE = 10;
var mCreate = function mCreate() {
  return [new Map()];
};
var mGet = function mGet(m, k) {
  for (var i = 0; i < m.length; ++i) {
    var v = m[i].get(k);

    if (v) {
      return v;
    }
  }
};

var squash = function squash(m) {
  var i = m.length - 1;
  var dst = new Map(m[i]);

  while (i) {
    --i;
    m[i].forEach(function (v, k) {
      dst.set(k, v);
    });
  }

  return [dst];
};

var mSet = function mSet(m, k, v) {
  if (m[0].size < SMALL_MAP_SIZE) {
    return [new Map(m[0]).set(k, v)].concat(m.slice(1));
  }

  if (m.length >= MAX_CHAIN_LENGTH) {
    m = squash(m);
  }

  return [new Map([[k, v]])].concat(m);
};
var mDel = function mDel(m, k) {
  if (m.length > 1) {
    m = squash(m);
  }

  var map = new Map(m[0]);
  map.delete(k);
  return [map];
};
var mForEach = function mForEach(map, cb) {
  var seen = new WeakSet();
  map.forEach(function (m) {
    m.forEach(function (v, k) {
      if (!seen.has(k)) {
        cb(v, k);
        seen.add(k);
      }
    });
  });
};
var mMerge = function mMerge(m, newM) {
  m = [].concat(newM, m);

  if (m.length > MAX_CHAIN_LENGTH) {
    m = squash(m);
  }

  return m;
};
var mToPrintable = function mToPrintable(m, toPrintableK, toPrintableV) {
  if (m.length > 1) {
    m = squash(m);
  }

  return Object.fromEntries([].concat(m[0].entries()).map(function (_ref) {
    var k = _ref[0],
        v = _ref[1];
    return [toPrintableK(k), toPrintableV(v)];
  }));
};

var ContextsMap = new Map();
var getContexts = function getContexts(scope) {
  if (!ContextsMap.has(scope)) {
    ContextsMap.set(scope, [useContextSelector.createContext(null), useContextSelector.createContext(null)]);
  }

  return ContextsMap.get(scope);
};

var isReactExperimental = !!(typeof process === 'object' && process.env.IS_REACT_EXPERIMENTAL) || !!React__default['default'].unstable_useMutableSource;

var useWeakMapRef = function useWeakMapRef() {
  var ref = React.useRef();

  if (!ref.current) {
    ref.current = new WeakMap();
  }

  return ref.current;
};

var warnAtomStateNotFound = function warnAtomStateNotFound(info, atom) {
  console.warn('[Bug] Atom state not found. Please file an issue with repro: ' + info, atom);
};

var updateAtomState = function updateAtomState(prevState, atom, partial, prevPromise, isNew) {
  var atomState = mGet(prevState, atom);

  if (!atomState) {
    if (!isNew && typeof process === 'object' && process.env.NODE_ENV !== 'production') {
      warnAtomStateNotFound('updateAtomState', atom);
    }

    atomState = {
      rev: 0,
      deps: new Map()
    };
  }

  if (prevPromise && prevPromise !== atomState.readP) {
    return prevState;
  }

  return mSet(prevState, atom, _extends({}, atomState, partial, {
    rev: atomState.rev + 1
  }));
};

var updateDependentsMap = function updateDependentsMap(prevState, state, dependentsMap) {
  mForEach(state, function (atomState, atom) {
    var _mGet;

    var prevDeps = (_mGet = mGet(prevState, atom)) == null ? void 0 : _mGet.deps;

    if (prevDeps === atomState.deps) {
      return;
    }

    var dependencies = new Set(atomState.deps.keys());

    if (prevDeps) {
      prevDeps.forEach(function (_, a) {
        var aDependents = dependentsMap.get(a);

        if (dependencies.has(a)) {
          // not changed
          dependencies.delete(a);
        } else {
          var newDependents = new Set(aDependents);
          dependentsMap.set(a, newDependents);
        }
      });
    }

    dependencies.forEach(function (a) {
      var aDependents = dependentsMap.get(a);
      var newDependents = new Set(aDependents).add(atom);
      dependentsMap.set(a, newDependents);
    });
  });
};

var addDependency = function addDependency(prevState, atom, dependency) {
  var nextState = prevState;
  var atomState = mGet(nextState, atom);
  var dependencyState = mGet(nextState, dependency);

  if (atomState && dependencyState) {
    var newDeps = new Map(atomState.deps).set(dependency, dependencyState.rev);
    nextState = mSet(nextState, atom, _extends({}, atomState, {
      deps: newDeps
    }));
  }

  return nextState;
};

var replaceDependencies = function replaceDependencies(prevState, atom, dependencies) {
  var nextState = prevState;
  var atomState = mGet(nextState, atom);

  if (!atomState) {
    if (typeof process === 'object' && process.env.NODE_ENV !== 'production') {
      warnAtomStateNotFound('replaceDependencies.atomState', atom);
    }

    return prevState;
  }

  nextState = mSet(nextState, atom, _extends({}, atomState, {
    deps: new Map([].concat(dependencies).map(function (a) {
      var _mGet$rev, _mGet2;

      return [a, (_mGet$rev = (_mGet2 = mGet(nextState, a)) == null ? void 0 : _mGet2.rev) != null ? _mGet$rev : 0];
    }))
  }));
  return nextState;
};

var readAtomState = function readAtomState(prevState, atom, setState, atomStateCache, force) {
  if (!force) {
    var _atomState = mGet(prevState, atom);

    if (_atomState) {
      return [_atomState, prevState];
    }

    _atomState = atomStateCache.get(atom);

    if (_atomState && [].concat(_atomState.deps.entries()).every(function (_ref) {
      var _mGet3;

      var a = _ref[0],
          r = _ref[1];
      return ((_mGet3 = mGet(prevState, a)) == null ? void 0 : _mGet3.rev) === r;
    })) {
      return [_atomState, mSet(prevState, atom, _atomState)];
    }
  }

  var isSync = true;
  var nextState = prevState;
  var error = undefined;
  var promise = undefined;
  var value = undefined;
  var dependencies = new Set();
  var flushDependencies = false;

  try {
    var promiseOrValue = atom.read(function (a) {
      if (dependencies) {
        dependencies.add(a);
      }

      if (!isSync) {
        setState(function (prev) {
          return addDependency(prev, atom, a);
        });
      }

      if (a !== atom) {
        var _readAtomState = readAtomState(nextState, a, setState, atomStateCache),
            _aState = _readAtomState[0],
            nextNextState = _readAtomState[1];

        if (isSync) {
          nextState = nextNextState;
        } else {
          // XXX is this really correct?
          setState(function (prev) {
            return mMerge(nextNextState, prev);
          });
        }

        if (_aState.readE) {
          throw _aState.readE;
        }

        if (_aState.readP) {
          throw _aState.readP;
        }

        return _aState.value;
      } // a === atom


      var aState = mGet(nextState, a) || atomStateCache.get(a);

      if (aState) {
        if (aState.readP) {
          throw aState.readP;
        }

        return aState.value;
      }

      return a.init; // this should not be undefined
    });

    if (promiseOrValue instanceof Promise) {
      promise = promiseOrValue.then(function (value) {
        var dependenciesToReplace = dependencies;
        dependencies = null;
        setState(function (prev) {
          return updateAtomState(replaceDependencies(prev, atom, dependenciesToReplace), atom, {
            readE: undefined,
            readP: undefined,
            value: value
          }, promise);
        });
      }).catch(function (e) {
        var dependenciesToReplace = dependencies;
        dependencies = null;
        setState(function (prev) {
          return updateAtomState(replaceDependencies(prev, atom, dependenciesToReplace), atom, {
            readE: e instanceof Error ? e : new Error(e),
            readP: undefined
          }, promise);
        });
      });
    } else {
      value = promiseOrValue;
      flushDependencies = true;
    }
  } catch (errorOrPromise) {
    if (errorOrPromise instanceof Promise) {
      promise = errorOrPromise.then(function () {
        setState(function (prev) {
          return readAtomState(mDel(prev, atom), atom, setState, atomStateCache)[1];
        });
      });
    } else if (errorOrPromise instanceof Error) {
      error = errorOrPromise;
    } else {
      error = new Error(errorOrPromise);
    }

    flushDependencies = true;
  }

  nextState = updateAtomState(nextState, atom, {
    readE: error,
    readP: promise,
    value: promise ? atom.init : value
  }, undefined, true);

  if (flushDependencies) {
    nextState = replaceDependencies(nextState, atom, dependencies);
    dependencies = null;
  } else {
    // add dependency temporarily
    dependencies.forEach(function (dependency) {
      nextState = addDependency(nextState, atom, dependency);
    });
  }

  var atomState = mGet(nextState, atom);
  isSync = false;
  return [atomState, nextState];
};

var updateDependentsState = function updateDependentsState(prevState, atom, setState, dependentsMap, atomStateCache) {
  var dependents = dependentsMap.get(atom);

  if (!dependents) {
    // no dependents found
    // this may happen if async function is resolved before commit.
    // not certain this is going to be an issue in some cases.
    return prevState;
  }

  var nextState = prevState;
  dependents.forEach(function (dependent) {
    if (dependent === atom || typeof dependent === 'symbol' || !mGet(nextState, dependent)) {
      return;
    }

    var _readAtomState2 = readAtomState(nextState, dependent, setState, atomStateCache, true),
        dependentState = _readAtomState2[0],
        nextNextState = _readAtomState2[1];

    var promise = dependentState.readP;

    if (promise) {
      promise.then(function () {
        setState(function (prev) {
          return updateDependentsState(prev, dependent, setState, dependentsMap, atomStateCache);
        });
      });
      nextState = nextNextState;
    } else {
      nextState = updateDependentsState(nextNextState, dependent, setState, dependentsMap, atomStateCache);
    }
  });
  return nextState;
};

var readAtom = function readAtom(state, readingAtom, setState, pendingStateMap, atomStateCache) {
  var prevState = pendingStateMap.get(state) || state;

  var _readAtomState3 = readAtomState(prevState, readingAtom, setState, atomStateCache),
      atomState = _readAtomState3[0],
      nextState = _readAtomState3[1];

  if (nextState !== prevState) {
    pendingStateMap.set(state, nextState);
  }

  return atomState;
};

var addAtom = function addAtom(id, addingAtom, dependentsMap) {
  var dependents = dependentsMap.get(addingAtom);
  var newDependents = new Set(dependents).add(id);
  dependentsMap.set(addingAtom, newDependents);
};

var delAtom = function delAtom(id, deletingAtom, dependentsMap, atomStateCache, addWriteThunk) {
  addWriteThunk(function (prev) {
    var nextState = prev;

    var del = function del(atom, dependent) {
      var dependents = dependentsMap.get(atom);
      var newDependents = new Set(dependents);
      newDependents.delete(dependent);

      if (!newDependents.size) {
        dependentsMap.delete(atom);
        var atomState = mGet(nextState, atom);

        if (atomState) {
          if (atomState.readP && typeof process === 'object' && process.env.NODE_ENV !== 'production') {
            console.warn('[Bug] saving atomState with read promise', atom);
          }

          atomStateCache.set(atom, atomState);
          nextState = mDel(nextState, atom);
          atomState.deps.forEach(function (_, a) {
            del(a, atom);
          });
        } else if (typeof process === 'object' && process.env.NODE_ENV !== 'production') {
          warnAtomStateNotFound('delAtom', atom);
        }
      } else {
        dependentsMap.set(atom, newDependents);
      }
    };

    del(deletingAtom, id);
    return nextState;
  });
};

var writeAtom = function writeAtom(writingAtom, update, setState, dependentsMap, atomStateCache, addWriteThunk) {
  var pendingPromises = [];

  var writeAtomState = function writeAtomState(prevState, atom, update) {
    var prevAtomState = mGet(prevState, atom);

    if (prevAtomState && prevAtomState.writeP) {
      var promise = prevAtomState.writeP.then(function () {
        addWriteThunk(function (prev) {
          return writeAtomState(prev, atom, update);
        });
      });
      pendingPromises.push(promise);
      return prevState;
    }

    var nextState = prevState;
    var isSync = true;

    try {
      var promiseOrVoid = atom.write(function (a) {
        var aState = mGet(nextState, a) || atomStateCache.get(a);

        if (!aState) {
          if (typeof process === 'object' && process.env.NODE_ENV !== 'production') {
            warnAtomStateNotFound('writeAtomState', a);
          }

          return a.init;
        }

        if (aState.readP && typeof process === 'object' && process.env.NODE_ENV !== 'production') {
          // TODO will try to detect this
          console.warn('Reading pending atom state in write operation. We need to detect this and fallback. Please file an issue with repro.', a);
        }

        return aState.value;
      }, function (a, v) {
        if (a === atom) {
          var partialAtomState = {
            readE: undefined,
            readP: undefined,
            value: v
          };

          if (isSync) {
            nextState = updateDependentsState(updateAtomState(nextState, a, partialAtomState), a, setState, dependentsMap, atomStateCache);
          } else {
            setState(function (prev) {
              return updateDependentsState(updateAtomState(prev, a, partialAtomState), a, setState, dependentsMap, atomStateCache);
            });
          }
        } else {
          if (isSync) {
            nextState = writeAtomState(nextState, a, v);
          } else {
            addWriteThunk(function (prev) {
              return writeAtomState(prev, a, v);
            });
          }
        }
      }, update);

      if (promiseOrVoid instanceof Promise) {
        pendingPromises.push(promiseOrVoid);
        nextState = updateAtomState(nextState, atom, {
          writeP: promiseOrVoid.then(function () {
            addWriteThunk(function (prev) {
              return updateAtomState(prev, atom, {
                writeP: undefined
              });
            });
          })
        });
      }
    } catch (e) {
      if (pendingPromises.length) {
        pendingPromises.push(new Promise(function (_resolve, reject) {
          reject(e);
        }));
      } else {
        throw e;
      }
    }

    isSync = false;
    return nextState;
  };

  var isSync = true;
  var writeResolve;
  var writePromise = new Promise(function (resolve) {
    writeResolve = resolve;
  });
  pendingPromises.unshift(writePromise);
  addWriteThunk(function (prevState) {
    if (isSync) {
      pendingPromises.shift();
    }

    var nextState = writeAtomState(prevState, writingAtom, update);

    if (!isSync) {
      writeResolve();
    }

    return nextState;
  });
  isSync = false;

  if (pendingPromises.length) {
    return new Promise(function (resolve, reject) {
      var loop = function loop() {
        var len = pendingPromises.length;

        if (len === 0) {
          resolve();
        } else {
          Promise.all(pendingPromises).then(function () {
            pendingPromises.splice(0, len);
            loop();
          }).catch(reject);
        }
      };

      loop();
    });
  }
};

var runWriteThunk = function runWriteThunk(lastStateRef, isLastStateValidRef, pendingStateMap, setState, contextUpdate, writeThunkQueue) {
  while (true) {
    if (!isLastStateValidRef.current || !writeThunkQueue.length) {
      return;
    }

    var thunk = writeThunkQueue.shift();
    var prevState = pendingStateMap.get(lastStateRef.current) || lastStateRef.current;
    var nextState = thunk(prevState);

    if (nextState !== prevState) {
      pendingStateMap.set(lastStateRef.current, nextState);
      Promise.resolve().then(function () {
        var pendingState = pendingStateMap.get(lastStateRef.current);

        if (pendingState) {
          pendingStateMap.delete(lastStateRef.current);
          contextUpdate(function () {
            setState(pendingState);
          });
        }
      });
    }
  }
};

var InnerProvider = function InnerProvider(_ref2) {
  var r = _ref2.r,
      c = _ref2.c,
      children = _ref2.children;
  var contextUpdate = useContextSelector.useContextUpdate(c);
  useIsoLayoutEffect(function () {
    if (isReactExperimental) {
      r.current = function (f) {
        contextUpdate(function () {
          scheduler.unstable_runWithPriority(scheduler.unstable_UserBlockingPriority, f);
        });
      };
    } else {
      r.current = function (f) {
        f();
      };
    }
  }, [contextUpdate]);
  return children;
};

var Provider = function Provider(_ref3) {
  var initialValues = _ref3.initialValues,
      scope = _ref3.scope,
      children = _ref3.children;
  var contextUpdateRef = React.useRef();
  var pendingStateMap = useWeakMapRef();
  var atomStateCache = useWeakMapRef();
  var dependentsMap = useWeakMapRef();

  var _useState = React.useState(function () {
    var initialState = mCreate();

    if (initialValues) {
      for (var _iterator = _createForOfIteratorHelperLoose(initialValues), _step; !(_step = _iterator()).done;) {
        var _step$value = _step.value,
            atom = _step$value[0],
            value = _step$value[1];
        initialState = mSet(initialState, atom, {
          value: value,
          rev: 0,
          deps: new Map()
        });
      }
    }

    return initialState;
  }),
      state = _useState[0],
      setStateOrig = _useState[1];

  var lastStateRef = React.useRef(state);
  var isLastStateValidRef = React.useRef(false);
  var setState = React.useCallback(function (setStateAction) {
    var pendingState = pendingStateMap.get(lastStateRef.current);

    if (pendingState) {
      if (typeof setStateAction !== 'function' && typeof process === 'object' && process.env.NODE_ENV !== 'production') {
        console.warn('[Bug] pendingState can only be applied with function update');
      }

      setStateOrig(pendingState);
    }

    isLastStateValidRef.current = false;
    setStateOrig(setStateAction);
  }, [pendingStateMap]);
  useIsoLayoutEffect(function () {
    var pendingState = pendingStateMap.get(state);

    if (pendingState) {
      pendingStateMap.delete(state);
      setState(pendingState);
      return;
    }

    updateDependentsMap(lastStateRef.current, state, dependentsMap);
    lastStateRef.current = state;
    isLastStateValidRef.current = true;
  });
  var writeThunkQueueRef = React.useRef([]);
  React.useEffect(function () {
    runWriteThunk(lastStateRef, isLastStateValidRef, pendingStateMap, setState, contextUpdateRef.current, writeThunkQueueRef.current);
  }, [state, setState, pendingStateMap]);
  var actions = React.useMemo(function () {
    return {
      add: function add(id, atom) {
        addAtom(id, atom, dependentsMap);
      },
      del: function del(id, atom) {
        delAtom(id, atom, dependentsMap, atomStateCache, function (thunk) {
          writeThunkQueueRef.current.push(thunk);
        });
      },
      read: function read(state, atom) {
        return readAtom(state, atom, setState, pendingStateMap, atomStateCache);
      },
      write: function write(atom, update) {
        return writeAtom(atom, update, setState, dependentsMap, atomStateCache, function (thunk) {
          writeThunkQueueRef.current.push(thunk);

          if (isLastStateValidRef.current) {
            runWriteThunk(lastStateRef, isLastStateValidRef, pendingStateMap, setState, contextUpdateRef.current, writeThunkQueueRef.current);
          } else {
            // force update (FIXME this is a workaround for now)
            setState(function (prev) {
              return mMerge(prev, mCreate());
            });
          }
        });
      }
    };
  }, [pendingStateMap, dependentsMap, atomStateCache, setState]);

  if (typeof process === 'object' && process.env.NODE_ENV !== 'production') {
    // eslint-disable-next-line react-hooks/rules-of-hooks
    useDebugState(state);
  }

  var _getContexts = getContexts(scope),
      ActionsContext = _getContexts[0],
      StateContext = _getContexts[1];

  return React.createElement(ActionsContext.Provider, {
    value: actions
  }, React.createElement(StateContext.Provider, {
    value: state
  }, React.createElement(InnerProvider, {
    r: contextUpdateRef,
    c: StateContext
  }, children)));
};

var atomToPrintable = function atomToPrintable(atom) {
  var _atom$debugLabel;

  return atom.key + ":" + ((_atom$debugLabel = atom.debugLabel) != null ? _atom$debugLabel : '<no debugLabel>');
};

var stateToPrintable = function stateToPrintable(state) {
  return mToPrintable(state, atomToPrintable, function (v) {
    return v.readE || v.readP || v.writeP || v.value;
  });
};

var useDebugState = function useDebugState(state) {
  React.useDebugValue(state, stateToPrintable);
};

var keyCount = 0; // global key count for all atoms

function atom(read, write) {
  var config = {
    key: ++keyCount
  };

  if (typeof read === 'function') {
    config.read = read;
  } else {
    config.init = read;

    config.read = function (get) {
      return get(config);
    };

    config.write = function (get, set, update) {
      set(config, typeof update === 'function' ? update(get(config)) : update);
    };
  }

  if (write) {
    config.write = write;
  }

  return config;
}

function assertContextValue(x, scope) {
  if (!x) {
    throw new Error("Please use <Provider" + (scope ? " scope=" + String(scope) : '') + ">");
  }
}

var isWritable = function isWritable(atom) {
  return !!atom.write;
};

function useAtom(atom) {
  var _getContexts = getContexts(atom.scope),
      ActionsContext = _getContexts[0],
      StateContext = _getContexts[1];

  var actions = useContextSelector.useContext(ActionsContext);
  assertContextValue(actions, atom.scope);
  var value = useContextSelector.useContextSelector(StateContext, React.useCallback(function (state) {
    assertContextValue(state);
    var atomState = actions.read(state, atom);

    if (atomState.readE) {
      throw atomState.readE; // read error
    }

    if (atomState.readP) {
      throw atomState.readP; // read promise
    }

    if (atomState.writeP) {
      throw atomState.writeP; // write promise
    }

    return atomState.value;
  }, [atom, actions]));
  React.useEffect(function () {
    var id = Symbol();
    actions.add(id, atom);
    return function () {
      actions.del(id, atom);
    };
  }, [actions, atom]);
  var setAtom = React.useCallback(function (update) {
    if (isWritable(atom)) {
      return actions.write(atom, update);
    } else {
      throw new Error('not writable atom');
    }
  }, [atom, actions]);
  React.useDebugValue(value);
  return [value, setAtom];
}

var useBridge = function useBridge(scope) {
  var _getContexts = getContexts(scope),
      ActionsContext = _getContexts[0],
      StateContext = _getContexts[1];

  var actions = useContextSelector.useBridgeValue(ActionsContext);
  var state = useContextSelector.useBridgeValue(StateContext);
  return React.useMemo(function () {
    return [actions, state];
  }, [actions, state]);
};
var Bridge = function Bridge(_ref) {
  var value = _ref.value,
      scope = _ref.scope,
      children = _ref.children;
  var actions = value[0],
      state = value[1];

  var _getContexts2 = getContexts(scope),
      ActionsContext = _getContexts2[0],
      StateContext = _getContexts2[1];

  return React.createElement(useContextSelector.BridgeProvider, {
    context: ActionsContext,
    value: actions
  }, React.createElement(useContextSelector.BridgeProvider, {
    context: StateContext,
    value: state
  }, children));
};

exports.Bridge = Bridge;
exports.Provider = Provider;
exports.atom = atom;
exports.useAtom = useAtom;
exports.useBridge = useBridge;
